from automon.log import Logging
from automon.integrations.datascience import Pandas


class Coinbase:
    def __init__(self, csv):
        self._log = Logging(name=Coinbase.__name__, level=Logging.DEBUG)

        self.csv = None
        self.df = None
        self.is_match = None
        self.type = Coinbase.__name__

        r = CoinbaseCSV(csv)
        if r.matches:
            self.is_match = True
            self.csv = csv
            self.df = r.df

    def __repr__(self):
        return self.csv

    def __eq__(self, other):
        if self.df.equals(other.df):
            return True
        return False


class CoinbaseCSV:
    def __init__(self, csv):
        self.csv = csv
        self.df = Pandas().read_csv(csv)
        self.matches = None

    def __repr__(self):
        return CoinbaseCSV.__name__

    def __eq__(self, other):
        if self.df.equals(other.df):
            return True
        return False



import unittest

from automon.integrations.requestsWrapper.rest import BaseRestClient

r = BaseRestClient()


class Client(unittest.TestCase):
    def test_get(self):
        r.get('https://1.1.1.1')
        r.requests.get('https://1.1.1.1')
        self.assertFalse(r.get('x://127.0.0.1'))


if __name__ == '__main__':
    unittest.main()



import unittest

from automon.integrations.instagram.client_browser import InstagramBrowserClient

c = InstagramBrowserClient(headless=False)


class InstagramClientTest(unittest.TestCase):
    if c.is_running():
        if c.authenticate():
            def test_authenticate(self):
                self.assertTrue(c.is_authenticated())
                c.browser.quit()


if __name__ == '__main__':
    unittest.main()



import unittest

from automon.integrations.nmap import Nmap


class NmapTest(unittest.TestCase):
    n = Nmap()

    def test_run(self):
        if self.n.isReady():
            self.assertTrue(Nmap().run('-n localhost', output=False))

    def test_Nmap(self):
        self.assertIsNotNone(Nmap('-n localhost', output=False))

    def test_output(self):
        n = Nmap()

        if self.n.isReady():
            self.assertTrue(n.run('-n 127.0.0.1'))
            self.assertTrue(len(n) > 0)

    def test_output_file(self):
        if self.n.isReady():
            self.assertTrue(Nmap().run('-n 127.0.0.1', cleanup=False))
            self.assertTrue(Nmap().run('-n 127.0.0.1', cleanup=True))
            self.assertTrue(Nmap('-n 127.0.0.1', cleanup=False))
            self.assertTrue(Nmap('-n 127.0.0.1', cleanup=True))

    def test_nmap(self):
        nmap = Nmap()

        if self.n.isReady():
            self.assertTrue(Nmap().nmap(command='-n localhost', output=False))
            self.assertTrue(Nmap().nmap(command='-n 127.0.0.1', output=False))
            self.assertTrue(Nmap().nmap(command='-n localhost', output=False))
            nmap.ready = False
            self.assertFalse(nmap.nmap(command='localhosta', output=False))

    def test_scan(self):
        nmap = Nmap()

        if self.n.isReady():
            self.assertTrue(Nmap().scan(command='-n localhost', output=False))
            self.assertTrue(Nmap().scan(command='-n localhost', output=False))
            nmap.ready = False
            self.assertFalse(nmap.scan(command='localhosta', output=False))

    def test_results(self):
        nmap = Nmap()

        if self.n.isReady():
            self.assertTrue(nmap.scan('-sn -n localhost', output=False))
            self.assertTrue(nmap.scan('-n -p 80,443 localhost', output=False))
            self.assertTrue(nmap.scan('-n -p 80,443 localhost localhost', output=False))


if __name__ == '__main__':
    unittest.main()



import unittest

from automon.integrations.neo4jWrapper.client import Neo4jClient


class Neo4jTest(unittest.TestCase):
    client = Neo4jClient()

    def test_cypher(self):
        if self.client.isConnected():
            c = Neo4jClient()

            # self.assertTrue(c.delete_all())
            self.assertTrue(c.create(prop='AAAA', value='BBBB'))
            self.assertTrue(c.cypher_run(f'MATCH (N) RETURN N'))
            self.assertTrue(c.merge(prop='AAAA', value='CCCC'))

            c.build_cypher(f'MATCH (N)')
            c.build_cypher(f'RETURN N')


if __name__ == '__main__':
    unittest.main()



from automon.log import Logging
from automon.helpers.osWrapper import environ

log = Logging(name='SplunkSoarConfig', level=Logging.DEBUG)


class SplunkSoarConfig:
    def __init__(self, host: str = None,
                 user: str = None,
                 password: str = None,
                 auth_token: str = None):
        """Splunk SOAR Config"""

        self.host = host or environ('SPLUNK_SOAR_HOST')
        self.user = user or environ('SPLUNK_SOAR_USER')
        self.password = password or environ('SPLUNK_SOAR_PASSWORD')
        self.auth = (self.user, self.password)
        self.auth_token = auth_token or environ('SPLUNK_SOAR_AUTH_TOKEN')

        self.headers = {'ph-auth-token': self.auth_token}

        if not self.host:
            log.warn(f'missing SPLUNK_SOAR_HOST')

    def __repr__(self):
        return f'{self.__dict__}'

    @property
    def is_ready(self) -> bool:
        if self.host:
            return True
        log.warn(f'bad config')
        return False



import unittest

from automon.log import Logging
from automon.integrations.sentryio.client import SentryClient


class CallbackTest(unittest.TestCase):
    sentry = SentryClient()
    log = Logging(name=__name__, level=Logging.DEBUG)
    log.callbacks.append(sentry)

    def test_sentry(self):
        self.assertTrue(self.log)
        self.assertTrue(self.log.info('test'))
        self.assertTrue(self.log.debug('test'))
        self.assertTrue(self.log.error('test'))
        self.assertTrue(self.log.warn('test'))
        self.assertTrue(self.log.critical('test'))


if __name__ == '__main__':
    unittest.main()



import unittest

from automon.integrations.splunk_soar import SplunkSoarClient

c = SplunkSoarClient()


class TestClient(unittest.TestCase):
    if c.is_connected():
        def test_list_app_run_generator(self):
            self.assertTrue([x for x in c.list_app_run_generator(page_size=1)])


if __name__ == '__main__':
    unittest.main()



import json

from automon.log import Logging
from automon.integrations.elasticsearch.metrics import Cluster
from automon.integrations.elasticsearch.config import ElasticsearchConfig
from automon.integrations.elasticsearch.client import ElasticsearchClient


class ElasticsearchJvmMonitor:
    def __init__(self, config: ElasticsearchConfig = None):
        self._log = Logging(ElasticsearchJvmMonitor.__name__, Logging.DEBUG)

        self._config = config if isinstance(config, ElasticsearchConfig) else ElasticsearchConfig()
        self._client = ElasticsearchClient(config) if isinstance(config, ElasticsearchConfig) else ElasticsearchClient()

        self._endpoint = self._client.config.ELASTICSEARCH_HOST

    def _get_all_stats(self):
        if self._client.connected():
            for endpoint in self._endpoint:
                try:
                    request = self._client.rest(f'{endpoint}/_nodes/stats?pretty')
                    request_json = request.text
                    return json.loads(request_json)
                except Exception as e:
                    self._log.error(f'Failed to get all stats: {e}')
                    return False

        return False

    def _get_all_jvm_metrics(self):
        if self._client.connected():
            try:
                return Cluster(self._get_all_stats())
            except Exception as e:
                self._log.error(f'Failed to get jvm metrics: {e}')

        return False

    def read_file(self, file):
        try:
            with open(file, 'rb') as stats:
                return json.load(stats)
        except Exception as e:
            self._log.error(f'Failed to read file: {e}')

    def get_metrics(self):
        if self._client.connected():
            try:
                return self._get_all_jvm_metrics()
            except Exception as e:
                self._log.error(f'Failed to get metrics: {e}')

        return False



import unittest

from automon.integrations.instagram.client import InstagramClient

c = InstagramClient()


class InstagramClientTest(unittest.TestCase):
    def test_authenticate(self):
        pass


if __name__ == '__main__':
    unittest.main()



import unittest

from datetime import datetime

from automon.integrations.sentryio import SentryClient
from automon.integrations.geoip import Geoip

from automon.log import Logging


class SentryClientTest(unittest.TestCase):
    def test_sentry(self):
        s = SentryClient()
        l = Logging()

        if s.isConnected():
            self.assertTrue(s.capture_exception(Exception(f'test capture_exception')))
            self.assertTrue(s.capture_message(f'test capture_message'))
            # self.assertTrue(s.capture_event('test capture_event', 'warning'))
            self.assertTrue(l.info(f'test log info'))
            self.assertTrue(l.debug(f'test log debug'))
            self.assertTrue(l.warning(f'test log warning'))
            self.assertTrue(l.error(f'test log error'))
            self.assertTrue(l.critical(f'test log critical'))


if __name__ == '__main__':
    unittest.main()



import unittest

from automon.integrations.splunk_soar import SplunkSoarClient

c = SplunkSoarClient()


class TestClient(unittest.TestCase):

    def test_create_artifact(self):
        if c.is_connected():
            id = c.create_container(label='testing', name='testing').id
            self.assertTrue(c.create_artifact(container_id=id))
        else:
            self.assertFalse(c.create_artifact(container_id=0))

    def test_create_container(self):
        if c.is_connected():
            self.assertTrue(c.create_container(label='testing', name='testing'))
        else:
            self.assertFalse(c.create_container(label='testing', name='testing'))

    def test_close_container(self):
        if c.is_connected():
            container = c.create_container(label='testing', name='testing')
            self.assertTrue(c.close_container(container.id))
        else:
            self.assertFalse(c.close_container())

    def test_delete_containers(self):
        if c.is_connected():
            container = c.create_container(label='testing', name='testing')
            self.assertTrue(c.delete_container(container_id=container.id))
        else:
            self.assertFalse(c.delete_container(container_id=0))

    def test_list_artifact(self):
        if c.is_connected():
            self.assertTrue(c.list_artifact())
        else:
            self.assertFalse(c.list_artifact())

    def test_get_container(self):
        if c.is_connected():
            container = c.create_container(label='testing', name='testing')
            self.assertTrue(c.get_container(container_id=container.id))
        else:
            self.assertFalse(c.get_container())

    def test_list_containers(self):
        if c.is_connected():
            self.assertTrue(c.list_containers())
        else:
            self.assertFalse(c.list_containers())

    def test_run_playbook(self):
        if c.is_connected():
            container = c.create_container(label='testing', name='testing')
            playbook = ''
            if playbook:
                self.assertTrue(c.run_playbook(
                    container_id=container.id,
                    playbook_id=playbook
                ))


if __name__ == '__main__':
    unittest.main()



import re
import os
import json
import base64
import urllib3
import requests

from .config import VdsConfig

from queue import Queue

from automon.log import Logging

# disable insecure ssl warnings
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)


class VdsLdapClient(object):
    """Not implemented"""
    pass


class VdsRestClient(object):
    def __init__(self, config: VdsConfig = None):
        """VDS REST client"""

        self.config = config or VdsConfig()
        self.records = Queue()

        # test connection
        self.connected = self.check_connection()

        self._log = Logging(name=VdsRestClient.__name__, level=Logging.INFO)

    @staticmethod
    def check_connection():
        """check if vds server is reacheable"""
        return False

    def search(self, ldap_filter: str = 'filter=cn=*', **kwargs):
        """search ldap"""
        return self._get(ldap_filter=ldap_filter, **kwargs)

    def search_insecure(self, ldap_filter: str = 'filter=cn=*', **kwargs):
        """search ldap, ignoring ssl certificates"""
        return self._get(ldap_filter=ldap_filter, verify=False, **kwargs)

    def search_cafile(self, ldap_filter: str = 'filter=cn=*', cafile: str = None, **kwargs):
        """search ldap, providing ssl certificate"""

        if os.path.exists(cafile):
            return self._get(ldap_filter=ldap_filter, verify=cafile, **kwargs)

        raise Exception(f'{cafile} not found')

    def search_all(self, ldap_filter: str = 'filter=cn=*', page_size: int = 10, cafile: str = None, **kwargs):
        """page through all results

        'cookie' Value
        cookie: Record the value in the ‘cookie’ field. This value defines a search parameter that will
            be used to display the next page of search results. If the value in the ‘cookie’ field is “null”,
            the last page of search results has been reached; there are no more pages to retrieve.


        page_size: number of records per page. max 500
        """

        initial_query = ldap_filter
        query = f'{initial_query}{self._paging(page_size=page_size)}'
        cookie = ''

        while 1:
            if not cookie:
                if cafile:
                    records = self.search_cafile(ldap_filter=query, cafile=cafile, **kwargs)
                else:
                    records = self.search_insecure(ldap_filter=query, **kwargs)
            else:
                paging_query = f'{query}{old_cookie}'
                if cafile:
                    records = self.search_cafile(ldap_filter=paging_query, cafile=cafile, **kwargs)
                else:
                    records = self.search_insecure(ldap_filter=paging_query, **kwargs)

            [self.records.put_nowait(x) for x in records['resources']]

            self._log.info(f'Records: {self.records.qsize()}')

            # need to fix cookie 'hostname' to reflect vds config server
            # otherwise get request fails to match hostname to ssl hostname
            # &cookie=localhost=NTEyNDIzODcw
            cookie = records['cookie']

            if cookie == 'null':
                break

            c = re.search('=(.*)', cookie)
            b64 = c.groups()[-1]

            old_cookie = f'&cookie={cookie}'

            # TODO: fix issue when using ssl verify
            # with ssl verify, paging, and given 'old_cookie', 'request.get' fails because request uses the IP
            # instead of the hostname. This fails the certificate check, because hostnames
            fix_cookie = f'&cookie={self.config.server}={b64}'

        return self.records

    def _paging(self, page_size: int = 100):
        """limit ldap by paging

        'totalResults' Value
        -1: The number of entries returned by the search exceeds the default size limit (1000 entries).
        -2: A next page of search results is available to be displayed. This value is displayed only when
            performing a paged search.
        -3: The last page of search results has been reached.
        Any other value: The total number of entries that were matched. This value is displayed only if
            the parameter SizeLimit is set to 0.


        page_size: If the PageSize option is used, ‘startIndex’, ‘count’ and ‘sizeLimit’ options will be ignored.
        """
        return f'&pageSize={page_size}'

    def _indexing(self, start_index: int = 0, count: int = 10, size_limit: int = 100):
        """limit ldap by indexing"""
        return f'&sizeLimit={size_limit}&startIndex={start_index}&count={count}'

    def _get(self, ldap_filter: str = None, **kwargs):
        """retrieve response from server"""
        if ldap_filter:
            ldap_filter = f'?{ldap_filter}'
        else:
            ldap_filter = ''

        basic_auth = base64.b64encode(f'{self.config.user}:{self.config.password}'.encode()).decode()

        headers = {
            'Authorization': f'Basic {basic_auth}'
        }

        url = f'{self.config.uri}/{self.config.basedn}{ldap_filter}'

        self._log.debug(f'query: {url}')

        try:
            r = requests.get(url, headers=headers, **kwargs)
        except Exception as e:
            raise Exception(e)

        [self._log.debug(f'results: {x}') for x in r.__dict__.items()]

        if r.status_code != 200:
            self._log.error(f'{url} {r.status_code} {r.reason}\n\n{r.content.decode()}')
            ldap_result = False
        else:
            self._log.debug(f'{r.status_code} {r.reason} {url}')
            ldap_result = json.loads(r.content.decode())

        return ldap_result



import os

from automon.log import Logging

from .robinhood import Robinhood
from .coinbase import Coinbase
from .other import Other


class CryptoAccounting:

    def __init__(self, csvs: str = None):
        self._log = Logging(name=CryptoAccounting.__name__, level=Logging.DEBUG)

        self.accounts = []

        self.supported_exchanges = [
            Robinhood,
            Coinbase
        ]

        if csvs:
            for path, folders, files in os.walk(csvs):
                for file in files:
                    extension = os.path.splitext(file)[-1].lower()
                    if '.csv' == extension:
                        file_path = os.path.join(path, file)
                        self.auto_detect(file_path)

    def auto_detect(self, csv):
        self._log.debug(f'supported exchanges: {self.supported_exchanges}')
        for exchange in self.supported_exchanges:
            self._log.debug(f'reading exchange: {exchange}')
            x = exchange(csv)
            if x.is_match:
                self._log.debug(f'exchange matched: {exchange}')
                if x not in self.accounts:
                    self._log.info(f'added {x}')
                    self.accounts.append(x)
                    return True
                self._log.debug(f'already added: {x}')
        else:
            o = Other(csv)
            if o not in self.accounts:
                self.accounts.append(o)
        return False

    def robinhood(self, csv):
        r = Robinhood(csv)
        if r not in self.accounts:
            self.accounts.append(r)
        return r

    def other(self, csv):
        o = Other(csv)
        if o not in self.accounts:
            self.accounts.append(o)
        return o

    def __repr__(self):
        return f'accounts: {len(self.accounts)}'



import unittest

from automon.integrations.minioWrapper.config import MinioConfig

c = MinioConfig()


class ConfigTest(unittest.TestCase):

    def test_MinioConfig(self):
        if c.is_ready:
            self.assertTrue(c.is_ready)
        else:
            self.assertFalse(c.is_ready)


if __name__ == '__main__':
    unittest.main()



import os
import slack
import random
import asyncio

from automon.log import Logging
from automon.helpers.nest_asyncioWrapper import AsyncStarter

from .config import ConfigSlack
from .bots import BotInfo
from .error import SlackError


class SlackAsyncClient(ConfigSlack):

    def __init__(self, token: str = ConfigSlack.slack_token, username: str = '',
                 channel: str = '', icon_emoji: str = None, icon_url: str = None):
        """Slack Async client
        """

        self._log = Logging(SlackAsyncClient.__name__, Logging.ERROR)

        self.token = ConfigSlack.slack_token or token
        self.client = slack.WebClient(token=token)

        try:
            self.connected = True if self.client.auth_test() else False
        except:
            self.connected = False

        # TODO: use token to get bot info
        self.username = ConfigSlack.slack_name or username or self._get_bot_info() or ''
        self.channel = ConfigSlack.SLACK_DEFAULT_CHANNEL or channel or ''
        self.icon_emoji = icon_emoji or ''
        self.icon_url = icon_url or ''

        # start consumer
        self._eventloop = AsyncStarter()
        self.queue = self._eventloop.queue
        self.task = self._eventloop.create_task(self._consumer())
        self._stop = False

        # TODO: integrate slacklog
        # self.slacklog = SlackLogging(token)

    def _get_bot_info(self):
        if not self.connected:
            return False

        try:
            name = BotInfo(self.client.bots_info()).name
            self._log.debug(f'Bot name: {name}')
            return name
        except Exception as e:
            error = SlackError(e)
            self._log.error(
                f'''[{self._get_bot_info.__name__}]\tCouldn't get bot name, missing permission: {error.needed}''',
                enable_traceback=False)
            return ''

        return ''

    def _run_until_complete(self):
        while True:
            if self.queue.qsize():
                asyncio.run(asyncio.sleep(0))
            else:
                break
        self._stop = True

    async def chat_postMessage(self, channel: str, text: str) -> slack.WebClient.chat_postMessage:

        if not self.connected:
            return False

        if not text:
            return SyntaxError

        await self.queue.put((channel, text))

    def files_upload(self, file, filename=None):
        """
        Upload file to slack

        note: username does nothing
        note: icon_emoji does nothing
        note: title takes precedent over filename
        note: title will display full string
        note: filename displays only filename, no extension

        :param file: path to file
        :param filename: (Optional) filename
        :return: Slack response or False
        """

        if not self.connected:
            return False

        # check if file exists
        if not os.path.isfile(file):
            self._log.error(f'File not found: {file}')
            self._log.error(f'Working dir: {os.getcwd()}')
            return False

        # get filename
        if not filename:
            filename = os.path.split(file)

        # get file size
        file_size = os.path.getsize(file)

        title = f'{filename} ({file_size}B'

        # TODO: add string validation for `filename`
        # TODO: find a way to change username for files_upload

        response = self.client.files_upload(
            file=file, filename=filename, title=title, username=self.username, channels=self.channel)

        assert response["ok"]

        self._log.debug(f'File uploaded: {file} ({file_size}B) ({self.username}')

        return response

    async def _consumer(self):

        burst = 0
        burst_max = 0
        retry = 0
        while self.connected:
            channel, text = await self.queue.get()

            if self._stop:
                break

            msg = f'{channel} @{self.username}: {text}'

            while self.connected:
                try:
                    self._log.debug(msg)
                    response = self.client.chat_postMessage(
                        text=text, channel=channel, username=self.username,
                        icon_emoji=self.icon_emoji, icon_url=self.icon_url)
                    assert response["ok"]

                    if not retry and self.queue.qsize() > 10:
                        # if a lot in the queue, try bursting
                        await asyncio.sleep(random.choice(range(2)))
                    else:
                        sleep = random.choice(range(4))
                        self._log.debug(f'sleeping {sleep}, queue size is {self.queue.qsize()}')
                        await asyncio.sleep(sleep)

                    self._log.debug(f'Burst: {burst}, Retry: {retry}, Queue {self.queue.qsize()}')

                    burst += 1
                    retry = 0

                    break
                except Exception as e:
                    if retry > 5:
                        break

                    retry += 1
                    burst_max = burst
                    error = SlackError(e)
                    self._log.error(
                        f'{self._consumer.__name__}\t{error.error}\t{msg}\tRetry: {retry}, Burst max: {burst_max}',
                        enable_traceback=False)
                    burst = 0



class XPaths(object):

    def __repr__(self):
        return 'Instagram XPaths'

    @property
    def login_user(self):
        return '/html/body/div[2]/div/div/div/div[1]/div/div/div/div[1]/section/main/div/div/div[1]/div[2]/form/div/div[1]/div/label/input'

    @property
    def login_pass(self):
        return '/html/body/div[2]/div/div/div/div[1]/div/div/div/div[1]/section/main/div/div/div[1]/div[2]/form/div/div[2]/div/label/input'

    @property
    def login_btn(self):
        return '/html/body/div[2]/div/div/div/div[1]/div/div/div/div[1]/section/main/div/div/div[1]/div[2]/form/div/div[3]/button'

    @property
    def profile_picture(self):
        return '/html/body/div[2]/div/div/div/div[1]/div/div/div/div[1]/section/nav/div[2]/div/div/div[3]/div/div[6]'

    @property
    def save_info(self):
        return '/html/body/div[2]/div/div/div/div[1]/div/div/div/div[1]/section/main/div/div/div/section/div/button'

    @property
    def save_info_not_now(self):
        return '/html/body/div[2]/div/div/div/div[1]/div/div/div/div[1]/section/main/div/div/div/div/button'

    @property
    def turn_on_notifications(self):
        return '/html/body/div[2]/div/div/div/div[2]/div/div/div[1]/div/div[2]/div/div/div/div/div[2]/div/div/div[3]/button[1]'

    @property
    def turn_on_notifications_not_now(self):
        return '/html/body/div[2]/div/div/div/div[2]/div/div/div[1]/div/div[2]/div/div/div/div/div[2]/div/div/div[3]/button[2]'


import os
import pandas

from io import StringIO
from time import time as epoch_time

from automon.log import Logging

from .series import Series
from .dataframe import DataFrame


class Pandas:

    def __init__(self):
        self._log = Logging(name=Pandas.__name__, level=Logging.DEBUG)

        self.df = None
        self.csv_name = None

    def Series(self, data, *args, **kwargs) -> Series:
        return Series(data, *args, **kwargs)

    def DataFrame(self, *args, **kwargs) -> DataFrame:
        return DataFrame(*args, **kwargs)

    def read_csv(self, file: str or StringIO, delimiter: str = None, **kwargs) -> pandas.read_csv:
        """read csv"""

        if type(file) == str:
            self.csv_name = file

        self.df = pandas.read_csv(file, delimiter=delimiter, **kwargs)
        self._log.info(f'imported {file}')
        return self.df

    def csv_from_string(self, csv, delimiter: str = None, **kwargs) -> pandas.read_csv:
        """read csv from string"""

        self.df = self.read_csv(StringIO(csv), delimiter=delimiter, **kwargs)
        self._log.info(f'imported csv string {len(csv) / 1024 / 1024} MB')
        return self.df

    def export_csv(self, file: str = None, overwrite: bool = False,
                   incremental: bool = False,
                   index: bool = False, **kwargs):
        """export to csv"""

        if self.csv_name:
            path, filename = os.path.split(self.csv_name)
        else:
            path = ''
            filename = f'df.csv'
        name, ext = os.path.splitext(filename)

        if file:
            csv_export = filename

        if incremental or not overwrite:
            csv_export = os.path.join(path, f'incremental-{name}-{int(epoch_time())}{ext}')

        with open(csv_export, 'w') as f:
            f.write(self.df.to_csv(index=index, **kwargs))
            return True

        return False

    def __repr__(self):
        return f'{self.df}'

    def __eq__(self, other):
        if self.df == other.df:
            return True
        return False



import asyncio
import traceback

from json import dumps
from asyncio import sleep

from automon.helpers.nest_asyncioWrapper import AsyncStarter
from automon.integrations.slackWrapper.client import SlackClient

from automon.integrations.slackWrapper.slack_formatting import Emoji, Chat, Format
from automon.log import Logging, INFO, ERROR, WARN, CRITICAL, DEBUG


class AsyncSlackLogging(SlackClient):

    def __init__(self, slack: SlackClient = None,
                 username: str = '',
                 channel: str = '',
                 change_user: bool = False,
                 change_icon: bool = False,
                 debug: bool = True):
        """Logging to Slack
        """

        self.slack = slack or SlackClient(username=username, channel=channel)
        self.slack.username = self.slack.username or username
        self.slack.channel = self.slack.channel or channel
        self.slack_username = self.slack.username
        self.slack_channel = self.slack.channel

        # start producer
        self._eventloop = AsyncStarter()
        self.queue = self._eventloop.queue
        self._task = self._eventloop.create_task(self._producer())
        self._stop = False

        # TODO: which takes precedent, icon or url?

        self._icon = Emoji.yay
        self._url = ''
        self.channel = self.slack.SLACK_DEFAULT_CHANNEL or '' \
            if not self.slack_username else f'#{self.slack_username}'
        self._format = Format.blockquote
        self._suffix = ''

        self._warn_icon = Emoji.warning
        self._warn_url = ''
        self.warn_channel = self.slack.SLACK_WARN_CHANNEL or '' \
            if not self.slack_username else f'#{self.slack_username}-warn'
        self._warn_format = Format.blockquote
        self._warn_suffix = ' warn'

        self._info_icon = Emoji.information_source
        self._info_url = ''
        self.info_channel = self.slack.SLACK_INFO_CHANNEL or '' \
            if not self.slack_username else f'#{self.slack_username}-info'
        self._info_format = Format.blockquote
        self._info_suffix = ' info'

        self._debug_icon = Emoji.magnifying_glass
        self._debug_url = ''
        self.debug_channel = self.slack.SLACK_DEBUG_CHANNEL or '' \
            if not self.slack_username else f'#{self.slack_username}-debug'
        self._debug_format = Format.blockquote
        self._debug_suffix = ' debugger'

        self._error_icon = Emoji.sos
        self._error_url = ''
        self.error_channel = self.slack.SLACK_ERROR_CHANNEL or '' \
            if not self.slack_username else f'#{self.slack_username}-error'
        self._error_format = Format.blockquote
        self._error_suffix = ' error'

        self._critical_icon = Emoji.sos
        self._critical_url = ''
        self.critical_channel = self.slack.SLACK_CRITICAL_CHANNEL or '' \
            if not self.slack_username else f'#{self.slack_username}-critical'
        self._critical_format = Format.blockquote
        self._critical_suffix = ' critical'

        self._test_icon = Emoji.yay
        self._test_url = ''
        self.test_channel = self.slack.SLACK_TEST_CHANNEL or '' \
            if not self.slack_username else f'#{self.slack_username}-test'
        self._test_format = Format.blockquote
        self._test_suffix = ' tester'

        if debug:
            self.warn_channel = self.debug_channel
            self.info_channel = self.debug_channel
            self.debug_channel = self.debug_channel
            self.error_channel = self.debug_channel
            self.critical_channel = self.debug_channel
            self.test_channel = self.debug_channel

            self._warn_icon = ''
            self._info_icon = ''
            self._debug_icon = ''
            self._error_icon = ''
            self._critical_icon = ''
            self._test_icon = ''

        if change_user is False:
            self._warn_suffix = ''
            self._info_suffix = ''
            self._debug_suffix = ''
            self._error_suffix = ''
            self._critical_suffix = ''
            self._test_suffix = ''

            self._warn_icon = ''
            self._info_icon = ''
            self._debug_icon = ''
            self._error_icon = ''
            self._critical_icon = ''
            self._test_icon = ''

        if change_icon is False:
            self._warn_icon = ''
            self._info_icon = ''
            self._debug_icon = ''
            self._error_icon = ''
            self._critical_icon = ''
            self._test_icon = ''

        # TODO: can you create an animation out of emojis?

    @staticmethod
    def _msg(msg: object) -> str or None:
        if isinstance(msg, dict):
            return dumps(msg, indent=4)
        if isinstance(msg, list) or isinstance(msg, tuple):
            try:
                new_msg = ''
                for m in msg:
                    new_msg += dumps(m, indent=4)
                return new_msg
            except Exception as _:
                return ''.join(str(msg))
        if msg is None:
            return Chat.none()

        return Chat.string(msg)

    async def _producer(self):
        while True:
            level, channel, text = await self.queue.get()

            if level is WARN:
                await self._warn(channel, text)
            elif level is INFO:
                await self._info(channel, text)
            elif level is DEBUG:
                await self._debug(channel, text)
            elif level is ERROR:
                await self._error(channel, text)
            elif level is CRITICAL:
                await self._critical(channel, text)
            elif level == Logging.TEST:
                await self._test(channel, text)

            if self._stop:
                break

            await sleep(0)

    async def _put_queue(self, level: int, channel: str, msg: str or None):
        await self.queue.put((level, channel, msg))

    def run_until_complete(self):
        while True:
            if self.queue.qsize():
                asyncio.run(sleep(0))
            else:
                break

        self._stop = True
        self.slack._run_until_complete()

    def close(self):
        self.run_until_complete()

    def default(self, msg: str) -> asyncio.tasks:
        asyncio.run(self._put_queue(WARN, self.channel, msg))

    def warn(self, msg: str) -> asyncio.tasks:
        asyncio.run(self._put_queue(WARN, self.warn_channel, msg))

    def info(self, msg: str) -> asyncio.tasks:
        asyncio.run(self._put_queue(INFO, self.info_channel, msg))

    def debug(self, msg: str) -> asyncio.tasks:
        asyncio.run(self._put_queue(DEBUG, self.debug_channel, msg))

    def error(self, msg: str = None) -> asyncio.tasks:
        asyncio.run(self._put_queue(ERROR, self.error_channel, msg))

    def critical(self, msg: str = None) -> asyncio.tasks:
        asyncio.run(self._put_queue(CRITICAL, self.critical_channel, msg))

    def test(self, msg: str or list or dict or tuple) -> asyncio.tasks:
        asyncio.run(self._put_queue(Logging.TEST, self.test_channel, msg))

    async def _warn(self, channel: str, msg: str or list or dict or tuple):
        self.set_slack_config(WARN)
        return await self.slack.chat_postMessage(channel, self._msg(msg))
        # self.set_slack_config()

    async def _info(self, channel: str, msg: str or list or dict or tuple):
        self.set_slack_config(INFO)
        return await self.slack.chat_postMessage(channel, self._msg(msg))

        # self.set_slack_config()

    async def _debug(self, channel: str, msg: str):
        self.set_slack_config(DEBUG)
        return await self.slack.chat_postMessage(channel, Chat.clean(msg))
        # self.set_slack_config()

    async def _error(self, channel: str, msg: str or list or dict or tuple = None,
                     msg_format: Format = None):
        self.set_slack_config(ERROR)

        tb = traceback.format_exc()
        tb = Chat.wrap(tb, Format.codeblock)
        if 'NoneType' not in tb:
            await self.slack.chat_postMessage(channel, tb)

        if msg:
            msg = self._msg(msg)
            if msg_format:
                msg = Chat.Format(msg, msg_format)
            return await self.slack.chat_postMessage(channel, self._msg(msg))
        # self.set_slack_config()

    async def _critical(self, channel: str, msg: str or list or dict or tuple = None):
        self.set_slack_config(CRITICAL)

        tb = traceback.format_exc()
        tb = Chat.wrap(tb, Format.codeblock)
        if 'NoneType' not in tb:
            await self.slack.chat_postMessage(channel, tb)

        if msg:
            msg = self._msg(msg)
            if not isinstance(msg, str):
                msg = Chat.wrap(msg, Format.codeblock)
            return await self.slack.chat_postMessage(channel, msg)
        # self.set_slack_config()

    async def _test(self, channel: str, msg: str):
        self.set_slack_config(Logging.TEST)
        await self.slack.chat_postMessage(channel, self._msg(msg))
        # self.set_slack_config()

    def set_slack_config(self, level=None):

        # TODO: every since moving to asyncio, this isn't used anymore and I don't know how to fix it yet

        if level is WARN:
            self.slack.username = f'{self.slack.username}{self._warn_suffix}'
            self.slack.icon_emoji = self._warn_icon
            self.slack.icon_url = self._warn_url
            self.slack.channel = self.warn_channel

        elif level is INFO:
            self.slack.username = f'{self.slack.username}{self._info_suffix}'
            self.slack.icon_emoji = self._info_icon
            self.slack.icon_url = self._info_url
            self.slack.channel = self.info_channel

        elif level is DEBUG:
            self.slack.username = f'{self.slack.username}{self._debug_suffix}'
            self.slack.icon_emoji = self._debug_icon
            self.slack.icon_url = self._debug_url
            self.slack.channel = self.debug_channel

        elif level is ERROR:
            self.slack.username = f'{self.slack.username}{self._error_suffix}'
            self.slack.icon_emoji = self._error_icon
            self.slack.icon_url = self._error_url
            self.slack.channel = self.error_channel

        elif level is CRITICAL:
            self.slack.username = f'{self.slack.username}{self._critical_suffix}'
            self.slack.icon_emoji = self._critical_icon
            self.slack.icon_url = self._critical_url
            self.slack.channel = self.critical_channel

        elif level == Logging.TEST:
            self.slack.username = f'{self.slack.username}{self._test_suffix}'
            self.slack.icon_emoji = self._test_icon
            self.slack.icon_url = self._test_url
            self.slack.channel = self.test_channel

        elif not level:
            self.slack.username = self.slack_username
            self.slack.channel = self.slack_channel
            self.slack.icon_emoji = ''
            self.slack.icon_url = ''



import unittest
import pandas as pd

from automon.integrations.datascience import Pandas

csv_empty = '''ASSET NAME	RECEIVED DATE	COST BASIS(USD)	DATE SOLD	PROCEEDS'''

csv_test = '''ASSET NAME	RECEIVED DATE	COST BASIS(USD)	DATE SOLD	PROCEEDS
BTC	10/08/20	10.00	10/10/20	10.37
BTC	10/10/20	99.00	10/12/20	100.63
BTC	10/13/20	201.01	10/19/20	202.47'''

data_empty = {}
data = {'data': [1, 2, 3]}

series_empty = pd.Series(data_empty).rename('data')
series = pd.Series(data).rename('data')
df = pd.DataFrame(series)
csv = df.to_csv()

with open('empty.csv', 'w') as f:
    f.write(csv_empty)

with open('test.csv', 'w') as f:
    f.write(csv_test)


class PandasTest(unittest.TestCase):
    pd = Pandas()

    def test_Pandas(self):
        self.assertTrue(Pandas)
        self.assertTrue(Pandas())

    def test_read_csv(self):
        self.assertTrue(Pandas().read_csv('empty.csv').empty)
        self.assertFalse(Pandas().read_csv('test.csv').empty)

    def test_csv_from_string(self):
        self.assertTrue(Pandas().csv_from_string(csv_empty).empty)
        self.assertFalse(Pandas().csv_from_string(csv_test).empty)

    def test_export_csv(self):
        test = Pandas()
        test.df = df
        self.assertTrue(test.export_csv())

    def test_Series(self):
        self.assertTrue(Pandas().Series(data=data_empty).empty)
        self.assertFalse(Pandas().Series(data=data).empty)

    def test_DataFrame(self):
        self.assertTrue(Pandas().DataFrame(data=series_empty).empty)
        self.assertFalse(Pandas().DataFrame(data=series).empty)


if __name__ == '__main__':
    unittest.main()



import unittest

from automon.liveliness import Liveliness


class LifelinessTest(unittest.TestCase):
    def test_Liveliness(self):
        self.assertTrue(Liveliness())
        self.assertFalse(Liveliness().up)
        self.assertFalse(Liveliness().check())

# if __name__ == '__main__':
#     unittest.main()



import unittest

from automon.integrations.splunk_soar import SplunkSoarClient

c = SplunkSoarClient()


class TestClient(unittest.TestCase):

    def test_run_playbook(self):
        if c.is_connected():
            container = c.create_container(label='testing', name='testing')
            playbook = ''
            c.run_playbook(
                container_id=container.id,
                playbook_id=playbook
            )

    def test_cancel_playbook(self):
        if c.is_connected():
            container = c.create_container(label='testing', name='testing')
            playbook = ''

            if playbook:
                run = c.run_playbook(
                    container_id=container.id,
                    playbook_id=playbook
                )
                get = c.get_playbook_run(playbook_run_id=run.playbook_run_id)
                cancel = c.cancel_playbook_run(run.playbook_run_id)


if __name__ == '__main__':
    unittest.main()



import unittest

from automon.integrations.seleniumWrapper.browser import SeleniumBrowser

browser = SeleniumBrowser()
browser.set_driver(browser.type.chrome_headless)
browser.set_resolution(device_type='web-large')


class SeleniumClientTest(unittest.TestCase):
    if browser.is_running():
        def test(self):
            while True:

                try:
                    if browser.get('http://bing.com'):
                        self.assertTrue(browser.save_screenshot())
                        self.assertTrue(browser.save_screenshot())
                        self.assertTrue(browser.save_screenshot(folder='./'))

                    browser.quit()
                    break

                except:
                    pass


if __name__ == '__main__':
    unittest.main()



import unittest

from ..config import InstagramConfig

config = InstagramConfig()


class InstagramConfigTest(unittest.TestCase):
    def test_config(self):
        if config.is_configured:
            self.assertTrue(config.is_configured)


if __name__ == '__main__':
    unittest.main()



import requests

from automon.log import Logging

log = Logging(name='RequestsConfig', level=Logging.DEBUG)


class RequestsConfig(object):

    def __init__(self):
        pass

    def is_ready(self):
        return f'{NotImplemented}'

    def __repr__(self):
        return f'{NotImplemented}'



import json
import minio

from datetime import datetime


class Bucket(minio.datatypes.Bucket):
    name: str
    creation_date: datetime

    def __init__(self, bucket: minio.datatypes.Bucket):
        self.__dict__.update(bucket.__dict__)

    def to_json(self):
        return json.dumps({k: f'{v}' for k, v in self.__dict__.items()})

    def __repr__(self):
        return f'{str(self.creation_date)[:19]} ({self.name})'

    def __str__(self):
        return f'{str(self.creation_date)[:19]} ({self.name})'



import mmap
import xmltodict
import pandas as pd

from automon.log import Logging
from pandas import DataFrame
from automon.helpers import Run
from automon.integrations.datascience import Pandas


class NmapResult(object):
    def __init__(self, file: str = None, run: Run = None, **kwargs):
        self._log = Logging(name=NmapResult.__name__, level=Logging.INFO)

        self.file = file
        self._run = run

        if file:

            with open(self.file, 'r+') as f:
                mm = mmap.mmap(f.fileno(), 0)
                xml = xmltodict.parse(mm)

            self.df = Pandas().DataFrame(data=xml, **kwargs)

            df = Pandas().DataFrame(data=xml, **kwargs)

            if 'host' in df.nmaprun:
                self.host = self._normalize_ports(df)
            else:
                self.host = None

            if 'hosthint' in df.nmaprun:
                self._hosthint = pd.json_normalize(df.loc['hosthint'])
            else:
                self._hosthint = None

            self._runstats = pd.json_normalize(df.loc['runstats'])
            self._scaninfo = pd.json_normalize(df.loc['scaninfo'])
            self._verbose = pd.json_normalize(df.loc['verbose'])

            # normalize output data

            self.command = df.loc['@args'][0]
            self.cmd = self.command
            self.time_start = df.loc['@start'][0]
            self.time_start_str = df.loc['@startstr'][0]

            self.hosts_up = self._runstats.loc[:, 'hosts.@up'][0]
            self.hosts_down = self._runstats.loc[:, 'hosts.@down'][0]
            self.hosts_total = self._runstats.loc[:, 'hosts.@total'][0]

            self.version = df.loc['@version'].iloc[0]

            self.elapsed = self._runstats.loc[:, 'finished.@elapsed'][0]
            self.summary = self._runstats.loc[:, 'finished.@summary'][0]
            self.time_finished = self._runstats.loc[:, 'finished.@time'][0]

            self._log.info(f'hosts up: {self.hosts_up}')
            self._log.info(f'hosts down: {self.hosts_down}')
            # self._log.info(f'hosts total: {self.hosts_total}')
            self._log.info(f'{self.summary}')
            self._log.info(f'finished {self.file} ({round(df.memory_usage().sum() / 1024, 2)} Kb)')

    def __repr__(self):
        msg = f'{self.summary} '

        if self.df.memory_usage().sum() / 1024 / 1024 / 1024 > 1:
            msg += f'({round(self.df.memory_usage().sum() / 1024 / 1024 / 1024, 2)} Gb)'
        else:
            msg += f'({round(self.df.memory_usage().sum() / 1024, 2)} Kb)'

        return msg

    def __len__(self):
        return int(self.df.memory_usage().sum())

    def _get_ports(self, df: DataFrame = None) -> DataFrame or False:

        if df is None:
            if self.host is not None:
                df_host = self.host
        else:
            df_host = pd.json_normalize(df.nmaprun.loc['host'])

        if 'ports.port' in df_host:
            s_ports = df_host.loc[:, 'ports.port']
            s_ports = s_ports.apply(lambda _list: pd.json_normalize(_list))
            return s_ports

        return False

    def _normalize_ports(self, df):

        df_host = pd.json_normalize(df.nmaprun.loc['host'])

        if 'ports.port' in df_host:
            s_ports = df_host.loc[:, 'ports.port']
            s_ports = s_ports.apply(lambda _list: pd.json_normalize(_list))
            scanned_ports = s_ports.apply(lambda x: x.loc[:, '@portid'])
            scanned_ports = scanned_ports.iloc[0].to_list()

            i = 0
            while i < s_ports.size:
                # get series index
                # use index to save port result to
                s_result = s_ports.iloc[i]

                for port in scanned_ports:
                    _row = s_result[s_result.loc[:, '@portid'] == port]
                    status = _row.loc[:, 'state.@state']
                    status.index = [i]

                    if port not in df_host:
                        df_host[port] = status
                    else:
                        df_host[port].update(status)

                    self._log.debug(f"{df_host.loc[:, ['address.@addr'] + [x for x in scanned_ports if x in df_host]]}")

                i += 1

            return df_host

    def hostnames(self):
        if 'hostnames.hostname.@name' in self.host:
            return self.host.loc[:, 'hostnames.hostname.@name']
        return False

    def ips(self):
        if 'address.@addr' in self.host:
            return self.host.loc[:, 'address.@addr']
        return False

    def ports(self, df: DataFrame = None):
        if df:
            return self._get_ports(df)
        return self._get_ports()



import os.path
import functools

from google.auth.transport.requests import Request
from google.oauth2.credentials import Credentials
from google_auth_oauthlib.flow import InstalledAppFlow
from googleapiclient.discovery import build
from googleapiclient.errors import HttpError

from automon.log import Logging

from .urls import PeopleUrls
from .config import PeopleConfig
from .results import ConnectionsResults

log = Logging(name='PeopleClient', level=Logging.DEBUG)


class PeopleClient:
    def __init__(self, client_id: str = None,
                 client_secret: str = None,
                 config: PeopleConfig = None):
        """Google People API Client"""

        self.config = config or PeopleConfig(
            client_id=client_id,
            client_secret=client_secret
        )

        self.contacts = []

    def __repr__(self):
        return f'{self.__dict__}'

    @property
    def _build(self) -> build:
        return build('people', 'v1', credentials=self.config.Credentials)

    @property
    def _client_config(self) -> dict:
        return self.config.oauth_dict()

    @property
    def _connections(self, *args, **kwargs):
        return self._people.connections(*args, **kwargs)

    @property
    def _people(self, *args, **kwargs):
        return self._build.people(*args, **kwargs)

    def _execute(self, func) -> ConnectionsResults:
        if self.isConnected():
            result = func.execute()
            return ConnectionsResults(result)

    def _list(self, *args, **kwargs) -> ConnectionsResults:
        return self._execute(
            self._connections.list(*args, **kwargs)
        )

    @property
    def client(self) -> build:
        return self._build

    def authenticate(self) -> bool:
        """Get Oauth"""
        if not self.config.isReady():
            return False

        creds = self.config.Credentials
        client_config = self._client_config
        scopes = self.config.scopes

        refresh_token = creds.refresh_token

        if refresh_token:
            try:
                creds.refresh(Request())
                return True
            except Exception as e:
                log.error(msg=f'authentication failed {e}', raise_exception=False)

        else:
            flow = InstalledAppFlow.from_client_config(
                client_config,
                scopes
            )
            new_creds = flow.run_local_server(port=0)
            # Save the credentials for the next run
            self.config.load_token(new_creds)
            return True

        return False

    def isConnected(self) -> bool:
        """Check if authenticated to make requests"""
        return self.authenticate()

    def _isConnected(func):
        @functools.wraps(func)
        def wrapped(self, *args, **kwargs):
            if self.authenticate():
                return func(self, *args, **kwargs)

        return wrapped

    @_isConnected
    def list_connections(
            self,
            resourceName: str = None,
            pageToken: str = None,
            pageSize: int = 2000,
            sortOrder: str = None,
            requestSyncToken: bool = None,
            syncToken: str = None,
            personFields: str = None,
            sources: str = None,
            **kwargs) -> ConnectionsResults:
        """

        :param resourceName: Required. The resource name to return connections for. Only people/me is valid
        :param pageToken:
        :param pageSize:
        :param sortOrder:
        :param requestSyncToken:
        :param syncToken:
        :param personFields:
        :param sources:
        :param args:
        :param kwargs:
        :return: PeopleResults
        """

        if not resourceName:
            resourceName = PeopleUrls().resourceName()

        if not personFields:
            personFields = PeopleUrls().personFields_toStr()

        return self._list(
            resourceName=resourceName,
            pageToken=pageToken,
            pageSize=pageSize,
            sortOrder=sortOrder,
            requestSyncToken=requestSyncToken,
            syncToken=syncToken,
            sources=sources,
            personFields=personFields,
            **kwargs)

    @_isConnected
    def list_connection_generator(self, pageToken: str = None, **kwargs):
        """Generator for paging through connections"""

        while True:
            result = self.list_connections(pageToken=pageToken, **kwargs)
            yield result
            pageToken = result.nextPageToken

            if not pageToken:
                break



import selenium
import selenium.webdriver.chrome.options

from selenium.webdriver import Chrome
from selenium.webdriver import Edge
from selenium.webdriver import Firefox
from selenium.webdriver import Ie
from selenium.webdriver import Proxy
from selenium.webdriver import Remote
from selenium.webdriver import Safari
from selenium.webdriver import WebKitGTK

from automon.log import Logging

from .config import SeleniumConfig

# fix for python36
try:
    import selenium.webdriver.chromium.options
    from selenium.webdriver import ChromiumEdge
    from selenium.webdriver import WPEWebKit
except:
    from selenium.webdriver import Chrome as ChromiumEdge
    from selenium.webdriver import Chrome as WPEWebKit


log = Logging(name='BrowserType', level=Logging.DEBUG)


class BrowserType(object):
    config: SeleniumConfig

    def __init__(self, config: SeleniumConfig):
        self.config = config
        self.webdriver = self.config.webdriver
        self.chromedriver = self.config.selenium_chromedriver_path

    def __repr__(self):
        return 'BrowserType'

    @property
    def chrome(self, options: list = None) -> Chrome:
        """Chrome"""
        log.info(f'Browser set as Chrome')

        chrome_options = selenium.webdriver.chrome.options.Options()

        if options:
            for arg in options:
                chrome_options.add_argument(arg)

        try:
            if self.chromedriver:
                return self.webdriver.Chrome(self.chromedriver, options=chrome_options)
            return self.webdriver.Chrome(options=chrome_options)
        except Exception as e:
            log.error(f'Browser not set. {e}', enable_traceback=False)

    @property
    def chrome_headless(self, options: list = None, **kwargs) -> Chrome:
        """Chrome headless

        chrome_options = Options()
        chrome_options.add_argument("--disable-extensions")
        chrome_options.add_argument("--disable-gpu")
        chrome_options.add_argument("--no-sandbox") # linux only
        chrome_options.add_argument("--headless")

        chrome_options.headless = True # also works

        """
        log.info(f'Browser set as Chrome Headless')

        chrome_options = selenium.webdriver.chrome.options.Options()
        chrome_options.headless = True

        if options:
            for arg in options:
                chrome_options.add_argument(arg)

        try:
            if self.chromedriver:
                return self.webdriver.Chrome(self.chromedriver, options=chrome_options, **kwargs)
            return self.webdriver.Chrome(options=chrome_options, **kwargs)
        except Exception as e:
            log.error(f'Browser not set. {e}', enable_traceback=False)

    @property
    def chromium_edge(self, options: list = None, **kwargs) -> ChromiumEdge:
        """Chromium"""
        log.info(f'Browser set as Chromium Edge')

        chromium_options = selenium.webdriver.chromium.options.ChromiumOptions()

        if options:
            for arg in options:
                chromium_options.add_argument(arg)

        try:
            if self.chromedriver:
                return self.webdriver.ChromiumEdge(self.chromedriver, options=chromium_options, **kwargs)
            return self.webdriver.ChromiumEdge(options=chromium_options, **kwargs)
        except Exception as e:
            log.error(f'Browser not set. {e}', enable_traceback=False)

    @property
    def edge(self, **kwargs) -> Edge:
        """Edge"""
        log.info(f'Browser set as Edge')
        return self.webdriver.Edge(**kwargs)

    @property
    def firefox(self, **kwargs) -> Firefox:
        """Firefox"""
        log.info(f'Browser set as Firefox')
        return self.webdriver.Firefox(**kwargs)

    @property
    def ie(self, **kwargs) -> Ie:
        """Internet Explorer"""
        log.info(f'Browser set as Internet Explorer')
        return self.webdriver.Ie(**kwargs)

    @property
    def opera(self):
        """Depreciated: Opera"""
        log.warn(f'Opera is depreciated')

    @property
    def proxy(self, **kwargs) -> Proxy:
        """Proxy"""
        log.info(f'Browser using proxy')
        return self.webdriver.Proxy(**kwargs)

    @property
    def phantomjs(self):
        """PhantomJS"""
        log.warn(f'PhantomJS not supported')

    @property
    def remote(self, **kwargs) -> Remote:
        """Remote"""
        log.info(f'Browser using remote browser')
        return self.webdriver.Remote(**kwargs)

    @property
    def safari(self, **kwargs) -> Safari:
        """Safari"""
        log.info(f'Browser set as Safari')
        return self.webdriver.Safari(**kwargs)

    @property
    def webkit_gtk(self, **kwargs) -> WebKitGTK:
        """WebKit GTK"""
        log.info(f'Browser set as WebKitGTK')
        return self.webdriver.WebKitGTK(**kwargs)

    @property
    def wpewebkit(self, **kwargs) -> WPEWebKit:
        """WPE WebKit"""
        log.info(f'Browser set as WPEWebKit')
        return self.webdriver.WPEWebKit(**kwargs)



from automon.log import Logging

from .datatypes import AbstractDataType

log = Logging('Artifact', level=Logging.CRITICAL)


class Artifact(AbstractDataType):
    name: str = None
    container: int = None
    id: int = None

    def __repr__(self):
        if self.name:
            return self.name
        return f'{self.to_dict()}'



import re

from automon.helpers import assertions
from automon.helpers.grok import GrokLegacy


def geolocation(string):
    """Parse any kind of geolocation data"""

    # TODO: parse any geolocation info (long, lat)

    pattern = [
        '([Long]{4}:[ ]?[0-9\.]*,[ ]?[Lat]{3}:[ ]?[0-9\.]*)'
    ]

    for p in pattern:
        c = re.compile(p)
        r = re.findall(c, string)

        if r:
            return r


class Magic:

    @staticmethod
    def magic_box(data: str) -> dict:
        """Do some grok magic on anything given and find everything"""

        all_matches = dict()
        grok = GrokLegacy.g

        for pattern in grok:

            try:
                compile_regex = re.compile(grok[pattern])  # compiled regex from g dict
                result = re.findall(compile_regex, data)  # regex search result

                if result:
                    list_results = []

                    if assertions.assert_list(result):
                        _list = result
                        for _item in _list:

                            if assertions.assert_tuple(_item):
                                _tuple = _item
                                for _item2 in _tuple:
                                    if len(_item2) > 0:
                                        list_results.append(_item2)

                            elif assertions.assert_string(_item):
                                if len(_item) > 0:
                                    list_results.append(_item)

                    if len(list_results) > 0:
                        all_matches[pattern] = list_results

            except Exception as err:
                # print('[!] Failed pattern: ' + grok_all_string[p] + ' => ' + str(err))
                pass

        return all_matches



import unittest
import types

from swiftclient.service import ClientException

from automon.integrations.swift import SwiftClient, SwiftError_, SwiftItem, SwiftPage, SwiftList
from automon.integrations.swift.config import SwiftConfig


class SwiftConfigTest(unittest.TestCase):

    def test_config(self):
        self.assertTrue(SwiftConfig())


class SwiftTest(unittest.TestCase):
    s = SwiftClient()

    def test_Swift(self):
        self.assertTrue(SwiftClient)
        self.assertTrue(self.s)

    def test_list(self):
        # self.assertEqual(self.s.list('test'), [])
        self.assertRaises(
            TypeError, self.s.list_container(container='0000000xxx'))
        self.assertRaises(
            TypeError, self.s.list_container(container='0000000xxx'))


class SwiftPageTest(unittest.TestCase):
    page = {
        'action': None,
        'container': None,
        'prefix': None,
        'success': True,
        'marker': None,
        'error': None,
        'traceback': None,
        'error_timestamp': None,
        'listing': []
    }
    error = {
        'action': None,
        'container': None,
        'prefix': None,
        'success': None,
        'marker': None,
        'error': ClientException,
        'traceback': None,
        'error_timestamp': None,
    }

    s = SwiftPage(page)
    e = SwiftPage(error)

    def test_SwiftPage(self):
        self.assertTrue(SwiftPage)
        self.assertTrue(self.s)
        self.assertTrue(self.s._dict())
        self.assertIsInstance(self.s._dict(), dict)
        self.assertEqual(self.s._dict(), self.page)
        self.assertEqual(self.e.error, ClientException)
        self.assertNotEqual(self.s.error, ClientException)
        self.assertTrue(self.s)
        self.assertIsInstance(self.s.list_gen(), types.GeneratorType)
        self.assertTrue(self.e.list_gen())
        self.assertTrue(f'{self.e}')
        self.assertTrue(f'{self.s}')


class SwiftErrorTest(unittest.TestCase):
    error = {
        'action': '',
        'container': '',
        'headers': '',
        'success': '',
        'error': '',
        'traceback': '',
        'error_timestamp': '',
        'response_dict': '',
    }

    def test_SwiftError(self):
        self.assertTrue(SwiftError_)
        self.assertTrue(SwiftError_(self.error))
        self.assertIsInstance(f'{SwiftError_(self.error)}', str)


class SwiftItemTest(unittest.TestCase):
    item = {
        'size': 0,
        'name': None,
        'hash': None,
        'etag': None,
        'content_type': None,
        'last_modified': None
    }
    item2 = {
        'size': 0,
        'name': 'test',
        'hash': None,
        'etag': None,
        'content_type': 'application/directory',
        'last_modified': None
    }

    def test_SwiftItem(self):
        self.assertTrue(SwiftItem)
        self.assertTrue(SwiftItem({}))
        self.assertFalse(SwiftItem(self.item).is_directory())
        self.assertEqual(SwiftItem(self.item).data(), self.item)
        self.assertFalse(SwiftItem(self.item).has_dir_marker())
        self.assertTrue(SwiftItem(self.item2).is_directory())
        self.assertEqual(SwiftItem(self.item2).data(), self.item2)
        self.assertTrue(SwiftItem(self.item2).filter('test'))
        self.assertFalse(SwiftItem(self.item2).filter('nottest'))
        self.assertTrue(f'{SwiftItem(self.item2)}')


if __name__ == '__main__':
    unittest.main()



import datetime

from automon.log import Logging

from .datatypes import AbstractDataType

log = Logging('Container', level=Logging.CRITICAL)


class Container(AbstractDataType):
    artifact_count: int = None
    start_time: datetime = None
    id: int = None
    name: str = None

    def __repr__(self):
        if self.name:
            return f'{self.name}'
        return f'{self.id}'



import unittest

from automon.helpers.subprocessWrapper import Run


class TestRun(unittest.TestCase):
    def test_pip(self):
        run = Run()

        run.run('ls | grep eric', shell=True, text=True)
        pass


if __name__ == '__main__':
    unittest.main()



from automon.log import Logging

log = Logging(name='PeopleUrls', level=Logging.ERROR)


class PeopleUrls:
    PEOPLE_API = 'https://people.googleapis.com'
    API_VER = 'v1'
    BASE_URL = f'{PEOPLE_API}/{API_VER}'
    RESOURCE_NAME = 'people'

    sortOrder = [
        'LAST_MODIFIED_ASCENDING',
        'LAST_MODIFIED_DESCENDING',
        'FIRST_NAME_ASCENDING',
        'LAST_NAME_ASCENDING'
    ]

    personFields = [
        'addresses',
        'ageRanges',
        'biographies',
        'birthdays',
        'calendarUrls',
        'clientData',
        'coverPhotos',
        'emailAddresses',
        'events',
        'externalIds',
        'genders',
        'imClients',
        'interests',
        'locales',
        'locations',
        'memberships',
        'metadata',
        'miscKeywords',
        'names',
        'nicknames',
        'occupations',
        'organizations',
        'phoneNumbers',
        'photos',
        'relations',
        'sipAddresses',
        'skills',
        'urls',
        'userDefined'
    ]

    def personFields_toStr(self):
        return ','.join(self.personFields)

    def resourceName(self) -> str:
        log.warn(msg=f'resourceName is deprecieated. Only people/me is valid.')
        return f'{self.RESOURCE_NAME}/me'



"""requires smidump"""

import os
import re
import yaml
import xmltodict
import subprocess

from automon.integrations.slackWrapper.slack_formatting import Chat
from automon.log import Logging

log = Logging(__name__, level=Logging.INFO)


class SmidumpFormat:
    """smidump output formats"""

    def __init__(self):
        """
        Format types:
            cm            : reverse engineered conceptual model
            corba         : corba IDL interface and OID definitions (JIDM)
            identifiers   : list of all identifiers
            imports       : recursive list of all imports
            jax           : Java AgentX sub-agent classes in separate files
            metrics       : metrics characterizing MIB modules
            mosy          : intermediate format generated by the mosy compiler
            netsnmp       : ANSI C code for the NET-SNMP package
            perl          : Perl MIB dictionaries
            python        : Python MIB dictionaries
            sming         : SMIng
            smiv1         : SMIv1 (RFC 1155, RFC 1212, RFC 1215)
            smiv2         : SMIv2 (RFC 2578, RFC 2579, RFC 2580)
            sppi          : SPPI (RFC 3159)
            scli          : ANSI C manager stubs for the gsnmp package
            svg           : SVG diagram
            tree          : structure of the OID tree
            types         : recursive list of all derived types
            sizes         : RFC 3416 PDU sizes excluding message / transport headers
            xml           : intermediate SMI XML exchange format
            xsd           : XML schema definitions
            compliances   : compliances with all included objects / notifications
            yang          : YANG format
            boilerplate   : generate security considerations boilerplate text
        """
        self.cm = 'cm'
        self.corba = 'corba'
        self.identifiers = 'identifiers'
        self.imports = 'imports'
        self.jax = 'jax'
        self.metrics = 'metrics'
        self.mosy = 'mosy'
        self.netsnmp = 'netsnmp'
        self.perl = 'perl'
        self.python = 'py'
        self.sming = 'sming'
        self.smiv1 = 'smiv1'
        self.smiv2 = 'smiv2'
        self.sppi = 'sppi'
        self.scli = 'scli'
        self.svg = 'svg'
        self.tree = 'tree'
        self.sizes = 'sizes'
        self.xml = 'xml'
        self.xsd = 'xsd'
        self.compliances = 'compliances'
        self.yang = 'yang'
        self.boilerplate = 'boilerplate'

    def keys(self):
        return self.__dict__.keys()

    def get(self, key):
        return self.__dict__.get(key)

    def dict(self):
        return self.__dict__

    def get_type(self, value):
        for key in self.keys():
            if self.get(key) is value:
                return key

    def __str__(self):
        return f'{self.__dict__}'


class MibFile:
    def __init__(self, device: str, path: str):
        self.path = path
        self.device = device
        self.filename = os.path.split(path)[-1]

        if os.path.isfile(path):
            log.debug(f'found MIB {path}')
        else:
            msg = f'{MibFile} {path} not found'
            log.error(msg)
            raise

    def __str__(self):
        return self.path

    def read(self):
        return open(self.path).read()


class MibMap(MibFile):
    def __init__(self, MibFile: MibFile, stdout: bytes):
        self._MibFile = MibFile
        self._data = stdout
        self.device = self._MibFile.device
        self.path = self._MibFile.path
        self.filename = self._MibFile.filename
        self.len = len(self._data)
        self.oids = []

        try:
            self.xml = xmltodict.parse(self._data)
            self.smi = self.xml['smi']
        except:
            self.xml = None
            self.smi = None

    def update(self, oids: str):
        self.oids = oids

    def __str__(self):
        # return self.data.decode()
        if self.smi:
            return f'{self.smi}'
        else:
            return f'{self._data}'


class Oid(dict):
    def __init__(self, device: str, filename: str, oid_name: str, oid: str, description: str, unused: list):
        self.device = device
        self.module_name = self.device
        self.oid = oid
        self.oid_name = oid_name
        self.name = filename
        self.filename = self.name
        self.mib_file = self.name
        self.description = description
        self._unused = unused

    def __str__(self):
        return f'{self.device}, {self.filename}, {self.oid}, {self.description}'

    def __eq__(self, other):
        try:
            return (self.oid, self.description) == (other.oid, other.description)
        except AttributeError:
            return NotImplemented

    def __lt__(self, other):
        return self.oid < other.oid


class RequiredSmidump:
    def __init__(self):
        self.requires = 'smidump'

    def check(self) -> True or False:
        if os.system(f'which {self.requires}') == 0:
            log.debug(f'{self.requires} OK')
            return True
        else:
            msg = (
                f'missing {self.requires}, '
                f'please install {self.requires}, '
                f'`apt install smitools`')
            log.error(msg)
            return False


class PrometheusModule(dict):
    def __init__(self):
        self.template = {
            'modules': {}
        }
        self._modules = self.template.get('modules')

    def add_module(self, module_name: str,
                   version: int = 2,
                   retries: int = 3,
                   max_repetitions: int = 25,
                   timeout: str = '5s') -> dict:
        """Not all options are here"""
        template = {
            f'{module_name}': {
                'walk': [],
                'version': version,
                'retries': retries,
                'max_repetitions': max_repetitions,
                'timeout': timeout,
                'auth': {
                    'community': 'public'
                }
            }
        }
        self._modules.update(template)

    def _get_module(self, module_name: str) -> dict:
        if module_name not in self._modules.keys():
            self.add_module(module_name)
        return self._modules.get(module_name)

    def _get_walk(self, module_name: str) -> dict:
        return self._get_module(module_name)['walk']

    def _get_auth(self, module_name: str) -> dict:
        return self._get_module(module_name)['auth']

    def add_oid(self, oid: Oid, module_name: str = None, max_length: int = 40):
        module_name = module_name if module_name else oid.module_name

        template = f'{oid.oid_name}'.ljust(max_length + 3)

        walk = self._get_walk(module_name)
        walk.append(f'{template} # {oid.oid} {oid.description}')
        self.template = self._modules

    def add_auth(self, module_name: str = None,
                 community_string: str = 'public',
                 username: str = '', password: str = '',
                 security_level: str = 'noAuthNoPriv' or 'authNoPriv' or 'authPriv',
                 auth_protocol: str = 'MD5' or 'SHA',
                 priv_protocol: str = 'DES' or 'AES',
                 priv_password: str = '', context_name: str = '') -> dict:
        template = {
            'community': community_string,
            'username': username,
            'password': password,
            'security_level': security_level,
            'auth_protocol': auth_protocol,
            'priv_protocol': priv_protocol,
            'priv_password': priv_password,
            'context_name': context_name
        }
        module = self._get_auth(module_name)
        module.update(template)

    def __str__(self):
        return f'{self.template}'


class PrometheusGeneratorConfig(PrometheusModule):
    def __init__(self):
        self.template = PrometheusModule()
        self.config = self.template.template
        self.path = None
        self.filename = f'generator.yml'

    def _yaml_cleaner(self, yml: yaml.dump):
        yml = yml.replace("'", '')
        yml = yml.replace('"', '')

        return yml

    def _export_to_yaml(self):
        yml = yaml.dump(self.config, width=1200)
        yml = self._yaml_cleaner(yml)

        return yml

    def save_config(self, path: str = None):
        if path:
            output_file = os.path.join(path, self.filename)
        else:
            output_file = self.filename

        with open(output_file, 'w') as f:
            yml = self._export_to_yaml()
            f.write(yml)

        return yml

    def __str__(self):
        return self.template


class MIBS:
    def __init__(self, path: str, device: str = None):

        self.path = path

        # Check for smidump
        self._required = RequiredSmidump().check()

        if device:
            self.devices = [device]
        else:
            self.devices = os.listdir(self.path)

        self.mibs = []
        self.maps = []
        self.oids = []

        self._max_format = 0

        for device in self.devices:

            # self.device_path = os.path.join(MIBS_dir, device)
            path_to_device_folder = os.path.join(self.path, device)

            if os.path.isfile(path_to_device_folder):
                continue

            for mib_filename in os.listdir(path_to_device_folder):

                path_to_mib_file = os.path.join(path_to_device_folder, mib_filename)

                if os.path.isfile(path_to_mib_file):
                    if re.search('.mib$', mib_filename):
                        mib = MibFile(device, path_to_mib_file)
                        self.mibs.append(mib)
                else:
                    log.error(f'not found MIB {path_to_mib_file}')

    def _mib_generator(self) -> MibFile:
        for mib in self.mibs:
            yield mib

    def _map_generator(self) -> MibMap:
        for map in self.maps:
            yield map

    def _iod_generator(self) -> Oid:
        for iod in self.oids:
            yield iod

    def _create_oid_tuple(self, object: dict) -> tuple:

        if not isinstance(object, dict):
            return

        # [
        # 	('@name', 'l3EcmpGroupTableCurrentEntries'),
        # 	('@oid', '0.1.1.7'),
        # 	('@status', 'mandatory'),
        # 	('syntax', OrderedDict([('type', OrderedDict([
        # 			('@module', ''),
        # 			('@name', 'Integer32')
        # 		]))])),
        # 	('access', 'readonly'),
        # 	('description', 'Number of ECMP Group table entries currently in use.')
        # ];

        used_keys = ['@name', '@oid', 'description']

        if '@name' and '@oid' and 'description' in object.keys():

            oid_name = object.get('@name')
            oid = object.get('@oid')
            description = Chat.multi_to_single_line(object.get('description'))

            _unused_keys = [x for x in object.keys() if x is not used_keys]
            _unused = [(x, object.get(x)) for x in _unused_keys]

            if oid and description:
                if len(oid) > self._max_format:
                    self._max_format = len(oid)

                if len(oid_name) > self._max_format:
                    self._max_format = len(oid_name)

                return oid_name, oid, description, _unused

    def _list_walk(self, object: list) -> list:

        tuple_oids = []

        if not isinstance(object, list) or None:
            return tuple_oids

        for item in object:

            if isinstance(item, dict):
                tuple_oids.extend(self._dict_walk(item))
            else:
                log.critical(f'{self._list_walk} {NotImplemented}')

        return tuple_oids

    def _dict_walk(self, object: dict) -> list:

        tuple_oids = []

        if not isinstance(object, dict) or None:
            return []

        for key in object.keys():
            value = object.get(key)

            if isinstance(value, dict):
                tuple_oids.extend(self._dict_walk(value))
            elif isinstance(value, list):
                tuple_oids.extend(self._list_walk(value))
            elif not isinstance(value, str) and value is not None:
                log.critical(f'{self._dict_walk} {NotImplemented}')

            oid_and_description = self._create_oid_tuple(object)

            if oid_and_description and oid_and_description not in tuple_oids:
                tuple_oids.append(oid_and_description)

        return tuple_oids

    def _xml_walk(self, xml: dict) -> list:

        tuple_oids = []

        if isinstance(xml, dict):
            tuple_oids.extend(self._dict_walk(xml))
        elif isinstance(xml, list):
            tuple_oids.extend(self._list_walk(xml))
        else:
            log.critical(f'{self._xml_walk} {NotImplemented}')

        return tuple_oids

    def generate_prometheus_config(self) -> open:

        for mibmap in self._map_generator():

            filename = mibmap.filename
            xml = mibmap.smi
            device = mibmap.device

            list_of_oid_tuples = self._xml_walk(xml)
            # log.debug(list_of_oid_tuples)

            if not list_of_oid_tuples:
                continue

            oids = []

            for tuple_ in list_of_oid_tuples:
                oid_name, oid, description, _unused = tuple_

                oid = Oid(device=device, filename=filename, oid_name=oid_name, oid=oid, description=description,
                          unused=_unused)
                oids.append(oid)

                if oid not in self.oids:
                    self.oids.append(oid)

            mibmap.update(oids)

        # now create prometheus config
        prometheus_config = PrometheusGeneratorConfig()
        self.oids.sort()
        for _oid in self.oids:
            prometheus_config.template.add_oid(_oid, max_length=self._max_format)

        return prometheus_config.save_config(path=self.path)

    def generate_map(self, extension: SmidumpFormat = None) -> MibMap:

        def smidump_run(cmd: str, create_file: bool = False) -> None:

            if not self._required:
                return False

            run = subprocess.run(cmd.split(), capture_output=True)
            data = run.stdout
            data_out = data.decode()
            data_err = run.stderr.decode()

            if data_err:
                log.error(f'{data_err}')

            map_ = MibMap(mib, data)
            self.maps.append(map_)

            if create_file:
                with open(map_.path, 'wb') as f:
                    f.write(map_._data)
                    log.debug(f'Wrote {map_.path} ({map_.len} B)')

            return True

        for mib in self._mib_generator():

            path = mib.path
            device = mib.device

            if not re.search('.mib$', path):
                continue

            preload = ''
            for x in self.mibs:
                if x.device is device:
                    preload = preload + f' --preload={x.path}'

            if extension:
                smidump_format = SmidumpFormat().get_type(extension)
                map_file = f'{mib}_map.{extension}'
                opts = f'--level=6 -s -m -k -f {smidump_format}'

                cmd = f'smidump {opts} {preload} {mib}'

                log.debug(cmd)
                #
                smidump_run(cmd)

            elif extension is None:

                for smidump_format in SmidumpFormat().keys():
                    extension = getattr(SmidumpFormat(), smidump_format)
                    map_file = f'{mib}_map.{extension}'
                    opts = f'--level=6 -s -m -k -f {smidump_format}'

                    cmd = f'smidump {opts} {preload} {mib}'
                    # cmd = f'{self.smidump} {self.opts} {mib}'

                    log.debug(cmd)
                    #
                    smidump_run(cmd)

# m = MIBS(device='opengear', path='path/to/mibs')
# m.generate_map(SmidumpFormat().xml)
# yml = m.generate_prometheus_config()
#
# log.info('Done')



import unittest

from automon.log.logger import Logging, LogStream


class LoggingTest(unittest.TestCase):
    log = Logging()

    def test_logger(self):
        self.assertTrue(self.log)

    def test_error(self):
        self.assertTrue(self.log.error('test'))

    def test_debug(self):
        self.assertTrue(self.log.debug('test'))

    def test_info(self):
        self.assertTrue(self.log.info('test'))

    def test_critical(self):
        self.assertTrue(self.log.critical('test'))

    def test_warn(self):
        self.assertTrue(self.log.warning('test'))
        with self.assertRaises(Exception):
            self.log.error(raise_exception=True)

    def test_now(self):
        self.assertTrue(self.log.now())

    def test_delta(self):
        self.assertTrue(self.log.uptime())


class LogStreamTest(unittest.TestCase):
    stream = LogStream()

    def test_stream(self):
        self.assertTrue(self.stream)

    def test_repr(self):
        self.assertFalse(f'{self.stream}')

    def test_flush(self):
        self.assertIsNone(self.stream.flush())

    def test_write(self):
        self.assertIsNone(self.stream.write('test'))


if __name__ == '__main__':
    unittest.main()



""" It's always learning, growing. Forever and forever

Written         : Eric Jaw
Version         : 1.0
Created         : 2017-05-27

"""

from threading import Thread
from queue import Queue
import asyncio


def start_loop(loop):
    asyncio.set_event_loop(loop)
    loop.run_forever()


def stop_loop(loop):
    # Canceling pending tasks and stopping the loop
    asyncio.gather(*asyncio.Task.all_tasks()).cancel()

    # Stopping the loop
    loop.stop()

    # Received Ctrl+C
    return loop.close()


def create_queue():
    """ A large thread-safe stack of items nad have multiple threading pick items for processing

    :return: Instantiate the Queue object
    """

    return Queue()


def job(queue_object):
    """ This job is to run

    :return:
    """

    # TODO: Finish building out queue for jobs

    while not queue_object().not_empty():
        # Assign variables
        # Assign job
        try:
            pass
        except:
            pass
        pass

    return


def thread_spawn(target=None, thread_count=1):
    """ Create the number of threading specified by "thread_count" and target

    :param target: The target function for the threading to spawn from
    :param thread_count: The default number of threading to spawn
    :return:
    """

    if target is None:
        return Exception('No target specified for thread spawning')
    else:
        for thread in range(thread_count):
            t = Thread(target=target)
            return t.start()


def plumbing():
    worker_loop = asyncio.new_event_loop()  # Get a reference to the event loop
    worker = Thread(target=start_loop, args=(worker_loop,))
    worker.start()



import os
import tempfile
import functools
import selenium

from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from urllib.parse import urlparse

from automon.log import Logging
from automon.helpers.dates import Dates
from automon.helpers.sleeper import Sleeper
from automon.helpers.sanitation import Sanitation

from .config import SeleniumConfig
from .browser_types import BrowserType

log = Logging(name='SeleniumBrowser', level=Logging.DEBUG)


class SeleniumBrowser(object):
    config: SeleniumConfig
    type: BrowserType

    def __init__(self, config: SeleniumConfig = None):
        """A selenium wrapper"""

        self.config = config or SeleniumConfig()
        self.driver = 'not set' or self.type.chrome_headless
        self.window_size = ''

        self.url = ''
        self.status = ''

    def __repr__(self):
        if self.url:
            return f'{self.browser.name} {self.status} {self.url} {self.window_size}'
        return f'{self.browser}'

    @property
    def type(self):
        return BrowserType(self.config)

    @property
    def browser(self):
        """alias to selenium driver"""
        return self.driver

    @property
    def by(self) -> By:
        """Set of supported locator strategies"""
        return selenium.webdriver.common.by.By()

    @property
    def keys(self):
        """Set of special keys codes"""
        return selenium.webdriver.common.keys.Keys

    @property
    def get_log(self, log_type: str = 'browser') -> list:
        """Gets the log for a given log type"""
        return self.browser.get_log(log_type)

    def _is_running(func) -> functools.wraps:
        @functools.wraps(func)
        def wrapped(self, *args, **kwargs):
            if self.browser != 'not set':
                return func(self, *args, **kwargs)
            log.error(f'Browser is not set!', enable_traceback=False)
            return False

        return wrapped

    def _screenshot_name(self, prefix=None):
        """Generate a unique filename"""

        title = self.browser.title
        url = self.browser.current_url
        hostname = urlparse(url).hostname

        hostname_ = Sanitation.ascii_numeric_only(hostname)
        title_ = Sanitation.ascii_numeric_only(title)
        timestamp = Dates.filename_timestamp()

        if prefix:
            prefix = Sanitation.safe_string(prefix)
            return f'{prefix}_{hostname_}_{title_}_{timestamp}.png'

        return f'{hostname_}_{title_}_{timestamp}.png'

    @_is_running
    def action_click(self, xpath: str, note: str = None) -> str or False:
        """perform mouse command"""
        try:
            click = self.find_element(value=xpath, by=self.by.XPATH)
            click.click()
            if note:
                log.debug(f'click: ({note}) {xpath}')
            else:
                log.debug(f'click: {xpath}')
            return click
        except Exception as e:
            log.error(f'failed to click: {xpath}, {e}', enable_traceback=False)
        return False

    @_is_running
    def action_type(self, key: str or Keys, secret: bool = False):
        """perform keyboard command"""
        try:
            actions = selenium.webdriver.common.action_chains.ActionChains(
                self.browser)
            actions.send_keys(key)
            actions.perform()

            if secret:
                key = f'*' * len(key)

            log.debug(f'type: {key}')
            return True
        except Exception as e:
            log.error(f'failed to type: {key}, {e}', enable_traceback=False)
        return False

    @_is_running
    def close(self):
        """close browser"""
        log.info(f'Browser closed')
        self.browser.close()

    @_is_running
    def find_element(
            self,
            value: str,
            by: By = By.ID,
            **kwargs):
        """find element"""
        return self.browser.find_element(value=value, by=by, **kwargs)

    @_is_running
    def find_xpath(self, value: str, by: By = By.XPATH, **kwargs):
        """find xpath"""
        return self.find_element(value=value, by=by, **kwargs)

    @_is_running
    def get(self, url: str, **kwargs) -> bool:
        """get url"""
        try:
            self.url = url
            self.browser.get(url, **kwargs)
            self.status = 'OK'
            log.debug(f'GET {url}, {kwargs}')
            return True
        except Exception as e:
            self.status = 'ERROR'
            log.error(f'Error getting {url}: {e}', enable_traceback=False)

        return False

    @_is_running
    def get_page(self, *args, **kwargs):
        """alias to get"""
        return self.get(*args, **kwargs)

    @_is_running
    def get_screenshot_as_png(self, **kwargs):
        """screenshot as png"""
        return self.browser.get_screenshot_as_png(**kwargs)

    @_is_running
    def get_screenshot_as_base64(self, **kwargs):
        """screenshot as base64"""
        return self.browser.get_screenshot_as_base64(**kwargs)

    @_is_running
    def is_running(self) -> bool:
        """browser is running"""
        return True

    @_is_running
    def quit(self) -> bool:
        """gracefully quit browser"""
        try:
            self.browser.close()
            self.browser.quit()
            self.browser.stop_client()
        except Exception as error:
            log.error(f'failed to quit browser. {error}')
            return False
        return True

    @_is_running
    def save_screenshot(
            self,
            filename: str = None,
            prefix: str = None,
            folder: str = None,
            **kwargs) -> bool:
        """save screenshot to file"""

        if not filename:
            filename = self._screenshot_name(prefix)

        if not folder:
            path = os.path.abspath(tempfile.gettempdir())
        else:
            path = os.path.abspath(folder)

        if not os.path.exists(path):
            os.makedirs(path)

        save = os.path.join(path, filename)

        log.info(f'Saving screenshot to: {save}')

        return self.browser.save_screenshot(save, **kwargs)

    def set_browser(self, browser: BrowserType) -> True:
        """set browser driver"""
        return self.set_driver(driver=browser)

    def set_driver(self, driver: BrowserType) -> True:
        """set driver"""
        if driver:
            self.driver = driver
        return True

    @_is_running
    def set_resolution(self, width=1920, height=1080, device_type=None) -> bool:
        """set browser resolution"""

        if device_type == 'pixel3':
            width = 1080
            height = 2160

        if device_type == 'web-small' or device_type == '800x600':
            width = 800
            height = 600

        if device_type == 'web-small-2' or device_type == '1024x768':
            width = 1024
            height = 768

        if device_type == 'web-small-3' or device_type == '1280x960':
            width = 1280
            height = 960

        if device_type == 'web-small-4' or device_type == '1280x1024':
            width = 1280
            height = 1024

        if device_type == 'web' or device_type == '1920x1080':
            width = 1920
            height = 1080

        if device_type == 'web-2' or device_type == '1600x1200':
            width = 1600
            height = 1200

        if device_type == 'web-3' or device_type == '1920x1200':
            width = 1920
            height = 1200

        if device_type == 'web-large' or device_type == '2560x1400':
            width = 2560
            height = 1400

        if device_type == 'web-long' or device_type == '1920x3080':
            width = 1920
            height = 3080

        if not width and not height:
            width = 1920
            height = 1080

        self.window_size = width, height

        try:
            self.browser.set_window_size(width, height)
        except Exception as error:
            log.error(f'failed to set resolution. {error}')
            return False
        return True

    def wait_for(
            self,
            value: str or list,
            by: By = By.XPATH,
            retries: int = 30,
            **kwargs) -> str or False:
        """wait for something"""
        retry = 1
        while True:
            try:
                if isinstance(value, list):
                    for each in value:
                        self.find_element(
                            by=by,
                            value=each,
                            **kwargs)
                        value = each
                        log.debug(f'found {by}: {value}')
                        return value
                else:
                    self.find_element(
                        by=by,
                        value=value,
                        **kwargs)
                    log.debug(f'found {by}: {value}')
                    return value
            except Exception as error:
                log.error(f'waiting for {by}: {value}, {error}',
                          enable_traceback=False)
                Sleeper.seconds(f'wait for', round(retry / 2))

            retry += 1

            if retry > retries:
                log.error(f'max wait reached', enable_traceback=False)
                break
        return False

    def wait_for_element(self, element: str or list, **kwargs) -> str or False:
        """wait for an element"""
        return self.wait_for(value=element, by=self.by.ID, **kwargs)

    def wait_for_xpath(self, xpath: str or list, **kwargs) -> str or False:
        """wait for an xpath"""
        return self.wait_for(value=xpath, by=self.by.XPATH, **kwargs)



import asyncio
import logging
import nest_asyncio

from automon.log import Logging

logging.getLogger("asyncio").setLevel(Logging.ERROR)


class AsyncStarter:

    def __init__(self) -> asyncio.get_event_loop:
        self._log = Logging(name=AsyncStarter.__name__, level=Logging.DEBUG)

        self.event_loop = asyncio.get_event_loop()
        self.maxqueue = 1000
        self.queue = asyncio.Queue(maxsize=self.maxqueue)
        self._nest = nest_asyncio.apply()
        asyncio.run(self._coro())

        self._finished = None

    async def _coro(self) -> asyncio:
        await asyncio.sleep(0)

    def run_until_complete(self):
        while True:
            if self.queue.qsize():
                asyncio.run(asyncio.sleep(0))
            else:
                self._finished = True
                break

    @staticmethod
    def sleep(seconds: int):
        asyncio.run(asyncio.sleep(seconds))

    def run(self) -> asyncio:
        asyncio.run(self._coro())

    def start(self) -> run:
        return self.run()

    def create_task(self, task):
        return self.event_loop.create_task(task)



import time
import random

from automon.log import Logging

log = Logging('Sleeper', level=Logging.INFO)


class Sleeper:

    @staticmethod
    def seconds(caller: object or str, seconds: int) -> time.sleep:
        """Sleep for this many seconds"""

        sleep = seconds
        if sleep < 2:
            log.info(f'[{Sleeper.seconds.__name__}] '
                     f'[{caller}] sleeping for {sleep} second')
        else:
            log.info(f'[{Sleeper.seconds.__name__}] '
                     f'[{caller}] sleeping for {sleep} seconds')
        return time.sleep(sleep)

    @staticmethod
    def minute(caller: object or str, sleep: int = 60) -> time.sleep:
        """Sleep for a minute"""

        log.info(f'[{Sleeper.minute.__name__}] '
                 f'[{caller}] sleeping for {sleep} seconds')
        return time.sleep(sleep)

    @staticmethod
    def within_a_minute(caller, sleep: int = None):
        """Sleep for a random minute"""

        sleep = sleep if isinstance(sleep, int) else \
            random.choice(range(1, 1 * 60))
        log.info(f'[{Sleeper.within_a_minute.__name__}] '
                 f'[{caller}] sleeping for {sleep} seconds')
        return time.sleep(sleep)

    @staticmethod
    def minutes(caller, minutes: int):
        """Sleep for this many minutes"""

        sleep = minutes * 60
        log.info(f'[{Sleeper.minutes.__name__}] '
                 f'[{caller}] sleeping for {sleep} minutes')
        return time.sleep(sleep)

    @staticmethod
    def hour(caller, hour: int = 1):
        """At some time within an hour, this will run"""

        sleep = hour if not hour else random.choice(
            range(1, hour * 60 * 60))
        log.info(f'[{Sleeper.hour.__name__}] '
                 f'[{caller}] sleeping for {sleep} seconds')
        return time.sleep(sleep)

    @staticmethod
    def hours(caller, hours):
        """Sleep for this many hours"""

        sleep = hours * 60 * 60
        log.info(f'[{Sleeper.hours.__name__}] '
                 f'[{caller}] sleeping for {hours} hours')
        return time.sleep(sleep)

    @staticmethod
    def day(caller, hours: int = 24):
        """At some time within 24 hours, this will run"""

        sleep = hours if not hours else random.choice(
            range(1, hours * 60 * 60))
        log.info(f'[{Sleeper.day.__name__}] '
                 f'[{caller}] sleeping for {sleep} seconds')
        return time.sleep(sleep)

    @staticmethod
    def daily(caller, hours: int = 24):
        """Sleep for one day"""

        sleep = hours if not hours else hours * 60 * 60
        log.info(f'[{Sleeper.daily.__name__}] '
                 f'[{caller}] sleeping for {sleep} seconds')
        return time.sleep(sleep)

    @staticmethod
    def time_range(caller, seconds: int):
        """Sleep for a random range
        """
        sleep = seconds if not seconds else random.choice(
            range(1, seconds))
        log.info(f'[{Sleeper.time_range.__name__}] '
                 f'[{caller}] sleeping for {sleep} seconds')
        return time.sleep(sleep)



import functools

from automon import Logging
from automon.integrations.seleniumWrapper.browser import SeleniumBrowser

from automon.helpers.sleeper import Sleeper
# from automon.integrations.minioWrapper import MinioClient

from .config import InstagramConfig
from .urls import Urls
from .xpaths import XPaths

log = Logging('InstagramClientBrowser', level=Logging.INFO)


class InstagramBrowserClient:
    login: str
    password: str
    config: InstagramConfig
    browser: SeleniumBrowser

    def __init__(self,
                 login: str = None,
                 password: str = None,
                 config: InstagramConfig = None,
                 headless: bool = True):
        """Instagram Browser Client"""
        self.config = config or InstagramConfig(login=login, password=password)
        self.browser = SeleniumBrowser()
        self.browser.set_browser(self.browser.type.chrome)

        if headless:
            self.browser.set_browser(self.browser.type.chrome_headless)

    def __repr__(self):
        return f'{self.__dict__}'

    def _is_running(func):
        @functools.wraps(func)
        def wrapped(self, *args, **kwargs):
            if self.config.is_configured:
                if self.browser.is_running():
                    return func(self, *args, **kwargs)
                return False
        return wrapped

    def _is_authenticated(func):
        @functools.wraps(func)
        def wrapped(self, *args, **kwargs):
            if self.browser.find_xpath(self.xpaths.profile_picture):
                return func(self, *args, **kwargs)
            return False

        return wrapped

    def _get_page(self, account):
        """ Get page
        """
        log.debug('[_get_page] getting {}'.format(account))

        page = 'https://instagram.com/{}'.format(account)
        browser = self.authenticated_browser
        return browser.get(page)

    def _get_stories(self, account):
        """ Retrieve story
        """
        story = 'https://www.instagram.com/stories/{}/'.format(account)
        num_of_stories = 0

        log.debug('[get_stories] {}'.format(story))

        browser = self.authenticated_browser
        browser.get(story)
        browser.browser.save_screenshot_to_minio(bucket_name='screenshots',
                                                 prefix='instagram/' + account)

        if 'Page Not Found' in browser.browser.title:
            log.debug('[get_stories] no stories for {}'.format(account))
            return num_of_stories

        Sleeper.seconds('instagram', 2)

        while True:
            try:
                self._next_story(browser)

                title = browser.browser.title
                if title == 'Instagram':
                    log.debug(
                        ('[get_stories] {} end of stories'.format(account)))
                    raise Exception
                num_of_stories += 1
                browser.save_screenshot_to_minio(bucket_name='screenshots',
                                                 prefix='instagram/' + account)
                Sleeper.seconds('watch the story for a bit', 1)
                browser.save_screenshot_to_minio(bucket_name='screenshots',
                                                 prefix='instagram/' + account)
            except Exception as error:
                # TODO: disable browser proxy when done
                log.debug(f'[get_stories] done: {account}, {error}')
                return num_of_stories

    def _next_story(self, authenticated_browser):
        """ Click next story button
        """

        xpaths = [
            '//*[@id="react-root"]/section/div/div/section/div[2]/div[1]/div/div/div[2]/div/div/button',
            '//*[@id="react-root"]/section/div/div/section/div[2]/button[2]'
        ]

        found_btn = False
        for xpath in xpaths:
            try:
                browser = authenticated_browser
                button = browser.browser.find_element_by_xpath(xpath)
                found_btn = True
                log.debug('[next_story] next story')
                return button.click()
            except Exception as error:
                log.error(f'{error}', enable_traceback=False)

        if not found_btn:
            # no more stories. exit
            log.debug('[_next_story] no more stories')
            raise Exception

    def run_stories(self, limit=None):
        """Run
        """

        log.debug('[login] {}'.format(self.login))

        self.authenticated_browser = self.authenticate()

        # if self.authenticated_browser:
        #
        #     count = 0
        #     if limit:
        #
        #         for account in self.following:
        #             hevlog.logging.debug(
        #                 '[runrun] [{}] {} session: {}'.format(self.authenticated_browser.browser.name,
        #                                                       self.authenticated_browser.browser.title,
        #                                                       self.authenticated_browser.browser.session_id))
        #
        #             num_of_stories = get_stories(self.authenticated_browser, account)
        #
        #             hevlog.logging.info('[{}] {} stories'.format(account, num_of_stories))
        #
        #             count = count + 1
        #             if count == limit:
        #                 return
        #
        #     Sleeper.hour('instagram')
        #     self.run_stories()

    @_is_running
    def authenticate(self):
        """Authenticate to Instagram
        """

        self.browser.get(self.urls.login_page)

        # user
        self.browser.wait_for_xpath(self.xpaths.login_user)
        self.browser.action_click(self.xpaths.login_user, 'user')
        self.browser.action_type(self.login)

        # password
        login_pass = self.browser.wait_for_xpath(self.xpaths.login_pass)
        self.browser.action_click(login_pass, 'login')
        self.browser.action_type(self.config.password, secret=True)

        # login
        login_btn = self.browser.wait_for_xpath(self.xpaths.login_btn)
        self.browser.action_click(login_btn, 'login button')

        # check for "save your login info" dialogue
        not_now = self.browser.wait_for_xpath(self.xpaths.save_info_not_now)
        self.browser.action_click(not_now, 'dont save login info')

        # check for "notifications" dialogue
        notifications_not_now = self.browser.wait_for_xpath(self.xpaths.turn_on_notifications_not_now)
        self.browser.action_click(notifications_not_now, 'no notifications')

        log.debug(
            f'[authenticated browser] [{self.browser.browser.name}] '
            f'{self.browser.browser.title} '
            f'session: {self.browser.browser.session_id}')

        if self.browser.wait_for_xpath(self.xpaths.profile_picture):
            return True

        return False

    @_is_running
    @_is_authenticated
    def is_authenticated(self):
        return True

    @_is_running
    def is_running(self) -> bool:
        return True

    @property
    def login(self) -> str:
        return self.config.login

    @property
    def urls(self):
        return Urls()

    @property
    def xpaths(self):
        return XPaths()


def get_page(authenticated_browser, account):
    """ Get page
    """
    # TODO: need to download page
    log.debug('[get_page] getting {}'.format(account))
    page = 'https://instagram.com/{}'.format(account)
    browser = authenticated_browser
    return browser.get(page)
